import { MagickInputFile, MagickOutputFile } from '.';
export declare type Command = (string | number)[];
export interface ExecuteConfig {
    inputFiles: MagickInputFile[];
    /** commands could be array form like [['convert', 'foo.png', 'bar.gif']] or CLI form like ['convert foo.png bar.gif'] */
    commands: ExecuteCommand;
}
export declare type ExecuteCommand = Command[] | string[] | string;
export interface ExecuteResult {
    outputFiles: MagickOutputFile[];
    errors?: any[];
}
/** execute first command in given config */
export declare function executeOne(config: ExecuteConfig): Promise<ExecuteResult>;
export interface ExecuteEvent {
    command: Command;
    took: number;
    id: number;
}
export interface ExecuteListener {
    afterExecute?(event: ExecuteEvent): void;
    beforeExecute?(event: ExecuteEvent): void;
}
export declare function addExecuteListener(l: ExecuteListener): void;
/**
 * Execute all commands in given config serially in order. Output files from a command become available as
 * input files in next commands. The execution result will contain all generated outputFiles. If same file name
 * is used later command output files will override previous ones. Example:
 *
 * ```ts
 * const {outputFiles} = await execute({
 *  inputFiles: [await buildInputFile('fn.png', 'image1.png')],
 *  commands: [
 *    ['convert', 'image1.png', "-bordercolor", "#ffee44", "-background", "#eeff55", "+polaroid", "image2.png"],
 *    // heads up: next command uses "image2.png" which was the output of previous command:
 *    ["convert", "image2.png", "-fill", "#997711", "-tint", "55"],
 *  ]
 * })
 * ```
 *
 * Alternatively it support CLI like command line instead of arrays:
 *
 * ```ts
 * const {outputFiles} = await execute({
 *   inputFiles: [await buildInputFile('fn.png', 'image1.png')],
 *   commands: [
 *     'convert image1.png -rotate 70 image2.gif',
 *     // heads up: next command uses 'image2.gif' which was the output of previous command:
 *     'convert image2.gif -scale 23% image3.jpg',
 *   ]
 * })
 * ```
 */
export declare function execute(config: ExecuteConfig): Promise<ExecuteResult>;
