"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _1 = require(".");
const p_map_1 = require("p-map");
/** execute first command in given config */
async function executeOne(config) {
    try {
        const command = _1.asCommand(config.commands)[0];
        const t0 = performance.now();
        executeListeners.forEach(listener => listener.beforeExecute({ command, took: performance.now() - t0, id: t0 }));
        const result = { outputFiles: await _1.Call(config.inputFiles, command.map(c => c + '')) };
        executeListeners.forEach(listener => listener.afterExecute({ command, took: performance.now() - t0, id: t0 }));
        return result;
    }
    catch (error) {
        return { outputFiles: [], errors: [error] };
    }
}
exports.executeOne = executeOne;
const executeListeners = [];
function addExecuteListener(l) {
    executeListeners.push(l);
}
exports.addExecuteListener = addExecuteListener;
/**
 * Execute all commands in given config serially in order. Output files from a command become available as
 * input files in next commands. The execution result will contain all generated outputFiles. If same file name
 * is used later command output files will override previous ones. Example:
 *
 * ```ts
 * const {outputFiles} = await execute({
 *  inputFiles: [await buildInputFile('fn.png', 'image1.png')],
 *  commands: [
 *    ['convert', 'image1.png', "-bordercolor", "#ffee44", "-background", "#eeff55", "+polaroid", "image2.png"],
 *    // heads up: next command uses "image2.png" which was the output of previous command:
 *    ["convert", "image2.png", "-fill", "#997711", "-tint", "55"],
 *  ]
 * })
 * ```
 *
 * Alternatively it support CLI like command line instead of arrays:
 *
 * ```ts
 * const {outputFiles} = await execute({
 *   inputFiles: [await buildInputFile('fn.png', 'image1.png')],
 *   commands: [
 *     'convert image1.png -rotate 70 image2.gif',
 *     // heads up: next command uses 'image2.gif' which was the output of previous command:
 *     'convert image2.gif -scale 23% image3.jpg',
 *   ]
 * })
 * ```
 */
async function execute(config) {
    const allOutputFiles = {};
    const allInputFiles = {};
    config.inputFiles.forEach(f => {
        allInputFiles[f.name] = f;
    });
    let allErrors = [];
    async function mapper(c) {
        const thisConfig = {
            inputFiles: Object.keys(allInputFiles).map(name => allInputFiles[name]),
            commands: [c],
        };
        const result = await executeOne(thisConfig);
        await p_map_1.default(result.outputFiles, async (f) => {
            allOutputFiles[f.name] = f;
            const inputFile = await _1.outputFileToInputFile(f);
            allInputFiles[inputFile.name] = inputFile;
        });
        allErrors = allErrors.concat(result.errors);
    }
    const commands = _1.asCommand(config.commands);
    await p_map_1.default(commands, mapper, { concurrency: 1 });
    return {
        outputFiles: Object.keys(allOutputFiles).map(name => allOutputFiles[name]),
        errors: allErrors,
    };
}
exports.execute = execute;
//# sourceMappingURL=execute.js.map